@@ -26,7 +26,7 @@ mydist <- function(data, k=20, distance = 2){
     dmtmp <- .C("dmEuclid", as.double(data), as.double(data), 
         as.integer(m), as.integer(m), as.integer(q), dm = double(m*k1), 
         cl = integer(m*k1), k = as.integer(k1), as.double(distance), 
-        as.double(rep(1,q)), dup=FALSE, PACKAGE = "kknn")
+        as.double(rep(1,q)), PACKAGE = "kknn")
     D <- matrix(dmtmp$dm, nrow = m, ncol = k1)[,-1, drop=FALSE]
     C <- matrix(dmtmp$cl + 1L, nrow = m, ncol = k1)[,-1, drop=FALSE]
     list(D, C)
